# _*_ coding: utf-8 _*_
from copy import deepcopy
from email import message_from_string, message_from_bytes

from cryptography import x509
from cryptography.hazmat.backends import default_backend as cryptography_backend
from cryptography.hazmat.bindings.openssl.binding import Binding as SSLBinding
from cryptography.hazmat.primitives import serialization


def sign_message(message, cert_signer, key_signer):
    # Get the message content. This could be a string, bytes or a message object
    passed_as_str = isinstance(message, str)

    if passed_as_str:
        message = message_from_string(message)

    passed_as_bytes = isinstance(message, bytes)
    if passed_as_bytes:
        message = message_from_bytes(message)

    # Extract the message payload without conversion, & the outermost MIME header / Content headers. This allows
    # the MIME content to be rendered for any outermost MIME type incl. multipart
    copied_msg = deepcopy(message)

    headers = {}
    # besides some special ones (e.g. Content-Type) remove all headers before encrypting the body content
    for hdr_name in list(copied_msg.keys()):
        if hdr_name in ["Content-Type", "MIME-Version", "Content-Transfer-Encoding"]:
            continue

        values = copied_msg.get_all(hdr_name)
        if values:
            del copied_msg[hdr_name]
            headers[hdr_name] = values

    content = copied_msg.as_bytes()

    # load cert & keys
    x509_cert = x509.load_pem_x509_certificate(cert_signer, cryptography_backend())
    private_key = serialization.load_pem_private_key(key_signer, None, cryptography_backend())

    # sign bytes and parse signed message
    signed_bytes = sign_bytes(content, x509_cert, private_key)
    signed_message = message_from_bytes(signed_bytes)

    # add original headers
    for hrd, values in headers.items():
        for val in values:
            try:
                signed_message.replace_header(hrd, str(val))
            except KeyError:
                signed_message.add_header(hrd, str(val))

    if passed_as_bytes:
        return signed_message.as_bytes()
    elif passed_as_str:
        return signed_message.as_string()
    else:
        return signed_message


def sign_bytes(byte_string, cert, key):
    """sign_bytes

    writen by kyrofa under Apache License 2.0 in:
    https://github.com/ros2/sros2/pull/129/commits/cfb4381fc1bc45a4f3ea9aa7e92f5228c08a2d04

    """
    # Using two flags here to get the output required:
    #   - PKCS7_DETACHED: Use cleartext signing
    #   - PKCS7_TEXT: Set the MIME headers for text/plain <- disabled to allow text/html
    # flags = 0
    flags = SSLBinding.lib.PKCS7_DETACHED
    # flags |= SSLBinding.lib.PKCS7_TEXT

    # Convert the byte string into a buffer for SSL
    bio_in = SSLBinding.lib.BIO_new_mem_buf(byte_string, len(byte_string))
    try:
        pkcs7 = SSLBinding.lib.PKCS7_sign(
            cert._x509, key._evp_pkey, SSLBinding.ffi.NULL, bio_in, flags)
    finally:
        # Free the memory allocated for the buffer
        SSLBinding.lib.BIO_free(bio_in)

    # PKCS7_sign consumes the buffer; allocate a new one again to get it into the final document
    bio_in = SSLBinding.lib.BIO_new_mem_buf(byte_string, len(byte_string))
    try:
        # Allocate a buffer for the output document
        bio_out = SSLBinding.lib.BIO_new(SSLBinding.lib.BIO_s_mem())
        try:
            # Write the final document out to the buffer
            SSLBinding.lib.SMIME_write_PKCS7(bio_out, pkcs7, bio_in, flags)

            # Copy the output document back to python-managed memory
            result_buffer = SSLBinding.ffi.new('char**')
            buffer_length = SSLBinding.lib.BIO_get_mem_data(bio_out, result_buffer)
            output = SSLBinding.ffi.buffer(result_buffer[0], buffer_length)[:]
        finally:
            # Free the memory required for the output buffer
            SSLBinding.lib.BIO_free(bio_out)
    finally:
        # Free the memory allocated for the input buffer
        SSLBinding.lib.BIO_free(bio_in)

    return output



# coding: utf-8

import os
from abc import ABCMeta
from abc import abstractmethod

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher
from cryptography.hazmat.primitives.ciphers import algorithms
from cryptography.hazmat.primitives.ciphers import modes


class BlockCipher:
    __metaclass__ = ABCMeta

    @abstractmethod
    def encrypt(self, data):
        return NotImplemented

    @property
    def session_key(self):
        return NotImplemented

    @property
    def parameters(self):
        return NotImplemented


class AES(BlockCipher):
    algorithm = None
    key_size = None
    block_size = 16

    def __init__(self, algorithm, mode, key_size):
        self.algorithm = algorithm
        self.mode = mode
        self.key_size = key_size
        self._session_key = os.urandom(self.key_size)
        self._iv = os.urandom(self.block_size)
        algorithm = algorithms.AES(self._session_key)
        mode = mode(self._iv)
        backend = default_backend()
        self._encryptor = Cipher(algorithm, mode, backend=backend).encryptor()

    @property
    def session_key(self):
        return self._session_key

    def encrypt(self, data):
        padded_data = self._pad(data, self.block_size)
        encrypted_content = (
            self._encryptor.update(padded_data.encode("utf-8")) + self._encryptor.finalize()
        )
        return {
            "content_type": "data",
            "content_encryption_algorithm": {
                "algorithm": self.algorithm,
                "parameters": self._iv,
            },
            "encrypted_content": encrypted_content,
        }

    @staticmethod
    def _pad(s, block_size):
        n = block_size - len(s) % block_size
        return s + n * chr(n)

    @property
    def parameters(self):
        return self._iv


def get_cipher(algorithm):
    _algorithms = {
        "aes128_cbc": (AES, (modes.CBC, 16)),
        "aes192_cbc": (AES, (modes.CBC, 24)),
        "aes256_cbc": (AES, (modes.CBC, 32)),
    }
    if algorithm in _algorithms:
        cipher, parameters = _algorithms[algorithm]
        return cipher(algorithm, *parameters)
    else:
        return None



""" Store the version here so:
# 1) we don't load dependencies by storing it in __init__.py
# 2) we can import it in setup.py for the same reason
# 3) we can import it into your module module
"""

__version_info__ = ('0', '5', '1')
__version__ = '.'.join(__version_info__)



# coding: utf-8

"""X509 Certificate API."""

import hashlib

from asn1crypto import cms
from asn1crypto import pem
from asn1crypto import x509

from cryptography.hazmat.primitives.serialization.pkcs12 import load_key_and_certificates
from cryptography.hazmat.backends import default_backend as cryptography_backend

from .pubkey import RSAPublicKeyCipher


class CertificateError(Exception):
    """Certificate has errors."""

    pass


class Certificate(object):
    """X509 certificates."""

    PEM_MARKERS = ("CERTIFICATE",)

    def __init__(self, der_string):
        """Initialize from a DER string.

        Args:
            der_string: a binary string containing the DER-encoded
                certificate.
        """
        self._cert = x509.Certificate.load(der_string)

    def __eq__(self, other):
        if isinstance(other, type(self)):
            return self.is_identical_to(other)
        else:
            return NotImplemented

    def __ne__(self, other):
        are_equal = self.__eq__(other)

        if are_equal is NotImplemented:
            return NotImplemented
        else:
            return not are_equal

    def __hash__(self):
        return hash(self.fingerprint())

    def __str__(self):
        return self._cert.debug()

    @classmethod
    def from_pem(cls, pem_string):
        """Read a single PEM-encoded certificate from a string.

        Args:
            pem_string: the certificate string.

        Returns:
            a Certificate object.
        """
        _, _, der_bytes = pem.unarmor(pem_string)
        return cls.from_der(der_bytes)

    @classmethod
    def from_der(cls, der_string):
        """Read a single DER-encoded certificate from a string.

        This is just an alias to __init__ to match from_pem().

        Args:
            der_string: the certificate string.

        Returns:
            a Certificate object.
        """
        return cls(der_string)

    @classmethod
    def from_pem_file(cls, pem_file):
        """Read a single PEM-encoded certificate from a file.

        Args:
            pem_file: the certificate file.

        Returns:
            a Certificate object
        """
        with open(pem_file, "rb") as pem_cert_file:
            return cls.from_pem(pem_cert_file.read())

    @classmethod
    def from_der_file(cls, der_file):
        """Read a single DER-encoded certificate from a file.

        Args:
            der_file: the certificate file.

        Returns:
            a Certificate object.
        """
        with open(der_file, "rb") as der_cert_file:
            return cls.from_der(der_cert_file.read())

    def to_der(self):
        """Get the DER-encoding of the certificate."""
        return self._cert.dump()

    def to_pem(self):
        """Get the PEM-encoding of the certificate."""
        return pem.armor(self.PEM_MARKERS[0], self.to_der())

    def is_identical_to(self, other_cert):
        """Returns True if this certificate is identical to |other_cert|."""
        return self.to_der() == other_cert.to_der()

    def to_asn1(self):
        """Get a copy of the ASN.1 representation of the certificate."""
        return x509.Certificate.load(self._cert.dump())

    @property
    def self_signed(self):
        """Is self signed?

        Returns:
            True or False.
        """
        return (
                self._cert["tbs_certificate"]["issuer"] == self._cert["tbs_certificate"]["subject"]
        )

    def fingerprint(self, hashfunc="sha1"):
        """Get the certificate fingerprint.

        Args:
            hashfunc: name of a hash function. Algorithms always present are
                'md5', 'sha1', 'sha224', 'sha256', 'sha384', and 'sha512'.
        Returns:
            a (binary) hash digest of the DER encoding.
        """
        h = hashlib.new(hashfunc)
        h.update(self._cert.dump())
        return h.digest()

    def key_hash(self, hashfunc="sha1"):
        """Get the certificate's public key hash.

        Args:
            hashfunc: name of a hash function. Algorithms always present are
                'md5', 'sha1', 'sha224', 'sha256', 'sha384', and 'sha512'.
        Returns:
            a (binary) hash digest of the public key.
        """
        h = hashlib.new(hashfunc)
        h.update(self._cert["tbs_certificate"]["subject_public_key_info"].dump())
        return h.digest()

    def _get_public_key_cipher(self):
        """
        :return:
            The PublicKey object for this certificate
        """
        algorithms = {RSAPublicKeyCipher.algo: RSAPublicKeyCipher}
        public_key_info = self._cert.public_key
        algorithm = public_key_info.algorithm
        if algorithm not in algorithms:
            return None
        t = algorithms[algorithm]
        return t(public_key_info)

    def recipient_info(self, session_key):
        cipher = self._get_public_key_cipher()
        if cipher is None:
            return None
        encrypted_key = cipher.encrypt(session_key)
        tbs_cert = self._cert["tbs_certificate"]
        # TODO: use subject_key_identifier when available
        return cms.RecipientInfo(
            name="ktri",
            value={
                "version": "v0",
                "rid": cms.RecipientIdentifier(
                    name="issuer_and_serial_number",
                    value={
                        "issuer": tbs_cert["issuer"],
                        "serial_number": tbs_cert["serial_number"],
                    },
                ),
                "key_encryption_algorithm": {
                    "algorithm": cipher.algo,
                    "parameters": cipher.parameters,
                },
                "encrypted_key": encrypted_key,
            },
        )


def certs_from_pem(pem_byte_string):
    """Read multiple PEM-encoded certificates from a byte string.

    Args:
        pem_byte_string: the certificate byte string.

    Yields:
        Certificate objects.
    """

    for _, _, der_bytes in pem.unarmor(pem_byte_string, multiple=True):
        yield Certificate.from_der(der_bytes)


def key_and_certs_from_pkcs12(data, password):
    """Load key, cert and additional certs from pkcs12 file

    PKCS12 is a binary format described in RFC 7292. It can contain
    certificates, keys, and more. PKCS12 files commonly have a pfx or
    p12 file suffix.

    Args:
        data (bytes-like) – The binary data.
        password (bytes-like) – The password to use to decrypt the data. None
            if the PKCS12 is not encrypted.

    :Returns:
        A tuple of (private_key, certificate, additional_certificates). private_key
            is a private key type or None, certificate is either the Certificate whose
            public key matches the private key in the PKCS 12 object or None, and
            additional_certificates is a list of all other Certificate instances in the
            PKCS12 object.
    """

    if isinstance(password, str):
        password = password.encode()

    return load_key_and_certificates(data, password, cryptography_backend())



import email
from email import message_from_bytes, message_from_string
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from smail.encrypt import encrypt_message
from smail.sign import sign_message


def _pop_headers(msg, blacklist=None):
    """ remove and return headers

    Attention: side effects - this will remove headers from `msg`
    Attention: duplicate headers are not supported at this point

    :param msg: `email.message.Message`
    :return: list of `tuples`
    """

    blacklisted_headers = set()
    blacklisted_headers.add('content-type')
    blacklisted_headers.add('mime-version')

    if blacklist:
        for item in blacklist:
            blacklisted_headers.add(item.lower())

    headers = []
    for header in msg.items():
        # print("processing: {} - {}".format(header[0], header[1]))
        if header[0].lower() in blacklisted_headers:
            continue

        if isinstance(header[0], Header):
            print("\n\n---\nFound a header!\n---\n\n")
        headers.append(header)
        msg.__delitem__(header[0])

    return headers


def sign_and_encrypt_message(message, cert_signer, key_signer, certs_recipients, algorithm="aes256_cbc"):
    # Get the message content. This could be a string, bytes or a message object
    passed_as_str = isinstance(message, str)
    if passed_as_str:
        message = message_from_string(message)

    passed_as_bytes = isinstance(message, bytes)
    if passed_as_bytes:
        message = message_from_bytes(message)

    popped_headers = _pop_headers(message)

    if isinstance(message, MIMEMultipart):
        payload = b''.join([x.as_bytes() for x in message.get_payload()])
    elif isinstance(message, MIMEText):
        # ensure that we have bytes
        payload = message.get_payload().encode()
    elif isinstance(message, str):
        payload = message.encode()
    else:
        payload = message.as_bytes()

    # print("---")
    # print("Payload")
    # print(type(payload))
    # print(payload)
    # print("---")

    payload_signed = sign_message(payload, cert_signer, key_signer)
    message_signed = email.message_from_bytes(payload_signed)

    # print("---")
    # print("Signed")
    # print(type(message_signed))
    # print(message_signed)
    # print("---")

    for header in popped_headers:
        try:
            message_signed.replace_header(header[0], str(header[1]))
        except KeyError:
            message_signed.add_header(header[0], str(header[1]))

    # print("---")
    # print("Signed+Headers")
    # print(type(message_signed))
    # print(message_signed)
    # print("---")

    message_signed_enveloped = encrypt_message(message_signed, certs_recipients, algorithm=algorithm)

    # print("---")
    # print("Signed+Enveloped")
    # print(type(message_signed_enveloped))
    # print(message_signed_enveloped)
    # print("---")

    if passed_as_bytes:
        return message_signed_enveloped.as_bytes()
    elif passed_as_str:
        return message_signed_enveloped.as_string()
    else:
        return message_signed_enveloped



# _*_ coding: utf-8 _*_
import subprocess


def get_cmd_output(args):
    try:
        result = subprocess.check_output(args, stderr=subprocess.STDOUT)

    except subprocess.CalledProcessError as err:
        raise Exception("Running shell command \"{}\" caused "
                        "error: {} (RC: {})".format(err.cmd, err.output, err.returncode))

    except Exception as err:
        raise Exception("Error: {}".format(err))

    return result.decode()



# Utilities for printing ASN.1 values


def bits_to_hex(bit_array, delimiter=":"):
    """Convert a bit array to a prettily formated hex string. If the array
    length is not a multiple of 8, it is padded with 0-bits from the left.
    For example, [1,0,0,1,1,0,1,0,0,1,0] becomes 04:d2.
    Args:
        bit_array: the bit array to convert
        delimiter: the delimiter to use (defaults to ":")
    Returns:
        the formatted hex string."""
    # Pad the first partial byte.
    partial_bits = len(bit_array) % 8
    pad_length = 8 - partial_bits if partial_bits else 0

    bitstring = "0" * pad_length + "".join(map(str, bit_array))
    byte_array = [int(bitstring[i: i + 8], 2) for i in range(0, len(bitstring), 8)]
    return delimiter.join(map(lambda x: "%02x" % x, byte_array))


def bytes_to_hex(byte_string, delimiter=":"):
    """Convert a bytestring to a prettily formated hex string: for example,
    '\x04\xd2' becomes 04:d2.
    Args:
        byte_string: the bytes to convert.
        delimiter: the delimiter to use (defaults to ":")
    Returns:
        the formatted hex string."""
    return delimiter.join([("%02x" % ord(b)) for b in byte_string])


def int_to_hex(int_value, delimiter=":"):
    """Convert an integer to a prettily formated hex string: for example,
    1234 (0x4d2) becomes 04:d2 and -1234 becomes ' -:04:d2'
    Args:
        int_value: the value to convert.
        delimiter: the delimiter to use (defaults to ":")
    Returns:
        the formatted hex string."""
    hex_string = "%x" % int_value
    ret = ""
    pos = 0
    # Accommodate for negative integers.
    if hex_string[0] == "-":
        ret += " -" + delimiter
        hex_string = hex_string[1:]
    # If the first digit is a half-byte, pad with a 0.
    remaining_len = len(hex_string) - pos
    hex_string = hex_string.zfill(remaining_len + remaining_len % 2)
    byte_values = [hex_string[i: i + 2] for i in range(pos, len(hex_string), 2)]
    return ret + delimiter.join(byte_values)


def append_lines(lines, wrap, buf):
    """Append lines to the buffer. If the first line can be appended to the last
    line of the buf without exceeding wrap characters, the two lines are merged.
    Args:
        lines: an iterable of lines to append
        wrap:  maximum number of characters per line. 0 or negative wrap means
               no limit.
        buf:   an iterable of lines to append to"""
    if not lines:
        return
    if not buf or 0 < wrap < len(buf[-1]) + len(lines[0]):
        buf += lines
    else:
        buf[-1] += lines[0]
        buf += lines[1:]


def wrap_lines(long_string, wrap):
    """Split the long string into line chunks according to the wrap limit and
    existing newlines.
    Args:
        long_string: a long, possibly multiline string
        wrap:        maximum number of characters per line. 0 or negative
                     wrap means no limit.
    Returns:
       a list of lines of at most |wrap| characters each."""
    if not long_string:
        return []
    if isinstance(long_string, bytes):
        long_string = long_string.decode()
    long_lines = long_string.split("\n")
    if wrap <= 0:
        return long_lines
    ret = []
    for line in long_lines:
        if not line:
            # Empty line
            ret += [line]
        else:
            ret += [line[i: i + wrap] for i in range(0, len(line), wrap)]
    return ret



from abc import ABCMeta
from abc import abstractmethod

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.asymmetric import rsa


class PublicKeyCipher(object):
    __metaclass__ = ABCMeta

    algo = None

    @abstractmethod
    def encrypt(self, session_key):
        return NotImplemented

    @property
    def parameters(self):
        return NotImplemented


class RSAPublicKeyCipher(PublicKeyCipher):
    algo = "rsa"

    def __init__(self, public_key_info):
        rsaparams = public_key_info["public_key"].native
        key = rsa.RSAPublicNumbers(rsaparams["public_exponent"], rsaparams["modulus"])
        backend = default_backend()
        self._cipher = key.public_key(backend)
        self._padding = padding.PKCS1v15()

    def encrypt(self, session_key):
        return self._cipher.encrypt(session_key, self._padding)

    @property
    def parameters(self):
        # AlgorithmIdentifier parameters is always NULL
        return None



# _*_ coding: utf-8 _*_

from base64 import b64encode
from copy import deepcopy
from email import message_from_string, message_from_bytes
from email.mime.text import MIMEText

from asn1crypto import cms

from .block import get_cipher
from .cert import certs_from_pem
from .print_util import wrap_lines


def __iterate_recipient_infos(certs, session_key):
    if isinstance(certs, (list, set, tuple)):
        for cert_file in certs:
            for cert in certs_from_pem(cert_file):
                recipient_info = cert.recipient_info(session_key)
                yield recipient_info
    else:
        for cert in certs_from_pem(certs):
            recipient_info = cert.recipient_info(session_key)
            yield recipient_info


def encrypt_message(message, certs_recipients, algorithm="aes256_cbc"):
    """
    Takes the contents of the message parameter, formatted as in RFC 2822 (type str or message), and encrypts them,
    so that they can only be read by the intended recipient specified by pubkey.
    :return: the new encrypted message (type str or message, as per input).
    """
    # Get the chosen block cipher
    block_cipher = get_cipher(algorithm)
    if block_cipher is None:
        raise ValueError("Unknown block algorithm")

    # Get the message content. This could be a string, bytes or a message object
    passed_as_str = isinstance(message, str)

    if passed_as_str:
        message = message_from_string(message)

    passed_as_bytes = isinstance(message, bytes)
    if passed_as_bytes:
        message = message_from_bytes(message)

    # Extract the message payload without conversion, & the outermost MIME header / Content headers. This allows
    # the MIME content to be rendered for any outermost MIME type incl. multipart
    copied_msg = deepcopy(message)

    headers = {}
    # besides some special ones (e.g. Content-Type) remove all headers before encrypting the body content
    for hdr_name in copied_msg.keys():
        if hdr_name in ["Content-Type", "MIME-Version", "Content-Transfer-Encoding"]:
            continue

        values = copied_msg.get_all(hdr_name)
        if values:
            del copied_msg[hdr_name]
            headers[hdr_name] = values

    content = copied_msg.as_string()
    recipient_infos = []

    for recipient_info in __iterate_recipient_infos(certs_recipients, block_cipher.session_key):
        if recipient_info is None:
            raise ValueError("Unknown public-key algorithm")
        recipient_infos.append(recipient_info)

    # Encode the content
    encrypted_content_info = block_cipher.encrypt(content)

    # Build the enveloped data and encode in base64
    enveloped_data = cms.ContentInfo(
        {
            "content_type": "enveloped_data",
            "content": {
                "version": "v0",
                "recipient_infos": recipient_infos,
                "encrypted_content_info": encrypted_content_info,
            },
        }
    )
    encoded_content = "\n".join(wrap_lines(b64encode(enveloped_data.dump()), 64))

    # Create the resulting message
    result_msg = MIMEText(encoded_content)
    overrides = (
        ("MIME-Version", "1.0"),
        (
            "Content-Type",
            "application/pkcs7-mime; smime-type=enveloped-data; name=smime.p7m",
        ),
        ("Content-Transfer-Encoding", "base64"),
        ("Content-Disposition", "attachment; filename=smime.p7m"),
    )

    for name, value in list(copied_msg.items()):
        if name in [x for x, _ in overrides]:
            continue
        result_msg.add_header(name, str(value))

    for name, value in overrides:
        if name in result_msg:
            del result_msg[name]
        result_msg[name] = value

    # add original headers
    for hrd, values in headers.items():
        for val in values:
            result_msg.add_header(hrd, str(val))

    # return the same type as was passed in
    if passed_as_bytes:
        return result_msg.as_bytes()
    elif passed_as_str:
        return result_msg.as_string()
    else:
        return result_msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open('README.rst') as f:
    README = f.read()

with open('LICENSE') as f:
    LICENSE = f.read()

with open('smail/version.py') as f:
    __version__ = ''
    exec(f.read())  # set __version__

test_requires = ['coverage', 'flake8', 'tox']
setup(
    name='python-smail',
    version=__version__,
    description='Simple S/MIME e-mails with Python3',
    long_description=README,
    url='https://gitlab.com/rhab/python-smail',
    author='Robert Habermann',
    author_email='mail@rhab.de',
    license='Apache License (2.0)',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries',
        'Topic :: Communications :: Email',
        'Topic :: Security :: Cryptography',
    ],
    keywords='smime cryptography email S/MIME encrypt sign',
    packages=find_packages(exclude=['tests', '*_test.py', 'test_*.py']),
    platforms=["all"],
    install_requires=['cryptography', 'asn1crypto'],
    setup_requires=['pytest-runner'],
    tests_require=test_requires,
    test_suite='tests',
    extras_require={
        'test': test_requires
    },
    zip_safe=False,
)



import os
from email import encoders
from email.header import Header
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate, formataddr

COMMASPACE = ', '


def make_msg(sender_addr, sender_name=None,
             recipients=None, subject=None,
             text=None, html=None,
             img_list=None, attachments=None):

    recipient_addr = []
    for recipient in recipients:
        if isinstance(recipient, tuple):
            recipient_addr.append(recipient[0])
        else:
            recipient_addr.append(recipient)

    msg_root = MIMEMultipart('mixed')
    msg_root['Date'] = formatdate(localtime=True)
    msg_root['From'] = formataddr((Header(sender_name, 'utf-8').encode(), sender_addr))
    msg_root['To'] = COMMASPACE.join(recipient_addr)
    msg_root['Subject'] = Header(subject, 'utf-8')
    msg_root.preamble = 'This is a multi-part message in MIME format.'

    msg_related = MIMEMultipart('related')
    msg_root.attach(msg_related)

    msg_alternative = MIMEMultipart('alternative')
    msg_related.attach(msg_alternative)

    msg_text = MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
    msg_alternative.attach(msg_text)

    msg_html = MIMEText(html.encode('utf-8'), 'html', 'utf-8')
    msg_alternative.attach(msg_html)

    if img_list:
        for i, img in enumerate(img_list):
            if isinstance(img, tuple):
                img_path = img[0]
                content_id = img[1]
            else:
                img_path = img
                content_id = 'image{}'.format(i)

            with open(img_path, 'rb') as fp:
                msg_image = MIMEImage(fp.read())
                msg_image.add_header('Content-ID', '<{}>'.format(content_id))
                msg_image.add_header('Content-Disposition', 'inline',
                                     filename=(Header(os.path.basename(img_path), 'utf-8').encode()))
                msg_related.attach(msg_image)

    if attachments:
        for attachment in attachments:
            fname = os.path.basename(attachment)

            with open(attachment, 'rb') as f:
                msg_attach = MIMEBase('application', 'octet-stream')
                msg_attach.set_payload(f.read())
                encoders.encode_base64(msg_attach)
                msg_attach.add_header('Content-Disposition', 'attachment',
                                      filename=(Header(fname, 'utf-8').encode()))
                msg_attach.add_header('Content-ID', '<%s>' % (Header(fname, 'utf-8').encode()))
                msg_root.attach(msg_attach)

    return msg_root
